/***************************************************************************
 *            main.cpp
 *
 *  Copyright  2007  Florent Berthaut
 *  florentberthaut@no-log.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <cmath>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <string>
#include <cwiid.h>
#include <vector>
#include <limits>
#include <bluetooth/bluetooth.h>
#include <alsa/asoundlib.h>

#define PI      3.14159265358979323
#define EVENT_NOTE 8  


using namespace std;


struct acc {
        unsigned char x;
        unsigned char y;
        unsigned char z;
};

struct stick {
        int valid;
        unsigned char x;
        unsigned char y;
        unsigned char max;
};

struct patch {

//wiimote's acc, nunchuk and ir active or not?
    bool acc;
    bool nunchuk;
    bool ir;

//MIDI conversion
    snd_seq_event_t ev_CWIID_BTN_2;
    snd_seq_event_t ev_CWIID_BTN_1;		
    snd_seq_event_t ev_CWIID_BTN_B;
    snd_seq_event_t ev_CWIID_BTN_A;
    snd_seq_event_t ev_CWIID_BTN_MINUS;
    snd_seq_event_t ev_CWIID_BTN_HOME;
    snd_seq_event_t ev_CWIID_BTN_LEFT;
    snd_seq_event_t ev_CWIID_BTN_RIGHT;
    snd_seq_event_t ev_CWIID_BTN_DOWN;
    snd_seq_event_t ev_CWIID_BTN_UP;
    snd_seq_event_t ev_CWIID_BTN_PLUS;

    snd_seq_event_t ev_CWIID_NUNCHUK_BTN_Z;
    snd_seq_event_t ev_CWIID_NUNCHUK_BTN_C;

    snd_seq_event_t ev_CWIID_ACC[3];

    snd_seq_event_t ev_CWIID_IR;

    snd_seq_event_t ev_CWIID_STICK;
};


int cptRoll=0,cptPitch=0;
float pbRoll=0,pbPitch=0;
float tabRoll[10],tabPitch[10];
int midiold, accmidiold[3];

snd_seq_t *seq_handle;
snd_seq_event_t previous;
int port_out_id, port_in_id;
int npfd;
//int pass=51, passstick=51, passnunacc=51;
struct pollfd *pfd;
uint8_t patch_id=0x00, pid;

cwiid_wiimote_t *wiimote = NULL;
struct acc_cal wm_cal, nc_cal;
struct cwiid_ir_mesg ir_data;
struct stick nc_stick;
struct stick cc_l_stick, cc_r_stick;
struct patch ppatch[16];

#define BATTERY_STR_LEN 14      /* "Battery: 100%" + '\0' */
#define LBLVAL_LEN 6
#define IP_MTU_SIZE 2048


bdaddr_t bdaddr;

//wiimote's midi bind
snd_seq_event_t* assign_midi_event(ifstream& bindings)
{
    string bind;
    int control, value, channel, maskchannel=0xF0, masktype=0x0F, type;
    snd_seq_event_t ev;

    bindings >> bind >> type;
    channel=type & masktype;
    type=(type & maskchannel)/16-1;

    if( type==SND_SEQ_EVENT_CONTROLLER )
    {
        bindings >> control >> value;

        snd_seq_ev_clear(&ev);
        snd_seq_ev_set_controller(&ev, (unsigned char)channel, (unsigned char)control, (unsigned char)value);
    }
    else if( type==SND_SEQ_EVENT_PGMCHANGE )
    {
        bindings >> control;

        snd_seq_ev_clear(&ev);
        snd_seq_ev_set_pgmchange(&ev, (unsigned char)channel, (unsigned char)control);
    }
    else if( type==EVENT_NOTE )
    {
        bindings >> control >> value;

        snd_seq_ev_clear(&ev);
        snd_seq_ev_set_noteon(&ev, (unsigned char)channel, (unsigned char)control, (unsigned char)value);
    }
    else
    {
        cerr << "Unsupported (by now) message for binding: " << bind << endl;
    }
 
    return &ev;
}

//wiimote's action towards incoming MIDI events
void midi_action(snd_seq_t *seq_handle)
{

  snd_seq_event_t *ev;

  do {
    snd_seq_event_input(seq_handle, &ev);

    switch (ev->type) {
      case SND_SEQ_EVENT_CONTROLLER: 
        break;
      case SND_SEQ_EVENT_PITCHBEND:
        break;
      case SND_SEQ_EVENT_NOTEON:
        break;        
      case SND_SEQ_EVENT_NOTEOFF: 
        break;        
      case SND_SEQ_EVENT_PGMCHANGE:
        // patch number change
        patch_id=ev->data.control.value % 17;
        fprintf(stderr,"Patch: %2d\n",patch_id);

        // Display patch number (hex) on the leds (2⁰ 2¹ 2² 2³)
        cwiid_set_led(wiimote,patch_id);
        if(patch_id == 16)
            fprintf(stderr,"Wiidi OFF\n");
        break;
    }
    snd_seq_free_event(ev);
  } while (snd_seq_event_input_pending(seq_handle, 0) > 0);

}

//wiimote's events callback
void cwiid_callback(cwiid_wiimote_t *wiimote, int mesg_count, union cwiid_mesg mesg_array[],struct timespec *timestamp)
{

    if( patch_id != 16 ) {
        int i;

    	char buffer[IP_MTU_SIZE];

        for (i=0; i < mesg_count; i++) 
	{
		snd_seq_event_t ev;


                switch (mesg_array[i].type) 
		{
                case CWIID_MESG_BTN: //button event
                        {


				if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_UP)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_UP;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				else if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_DOWN)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_DOWN;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				else if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_LEFT)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_LEFT;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				else if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_RIGHT)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_RIGHT;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				else if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_A)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_A;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				else if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_B)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_B;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				else if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_MINUS)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_MINUS;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				else if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_PLUS)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_PLUS;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				else if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_HOME)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_HOME;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				else if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_1)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_1;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				else if(mesg_array[i].btn_mesg.buttons & CWIID_BTN_2)
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_BTN_2;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

                if( previous.type==SND_SEQ_EVENT_NOTEON )
                {
                    //sends NOTE OFF if the previous event was a NOTE ON
					snd_seq_ev_clear(&ev);
					snd_seq_ev_set_noteoff(&ev, previous.data.note.channel, previous.data.note.note, 127);
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);

                    snd_seq_ev_clear(&previous);
                    previous=ev;
                }

			}
                        break;
                case CWIID_MESG_ACC:
			{
                if( ppatch[patch_id].acc)
                {
    				//we get the values

	    			float a_x, a_y, a_z, a;
		    		float roll, pitch;
                    int midivalue;
			    	struct cwiid_acc_mesg *mesg = &mesg_array[i].acc_mesg;

    				a_x = ((double)mesg->acc[CWIID_X] - wm_cal.zero[CWIID_X]) / (wm_cal.one[CWIID_X] - wm_cal.zero[CWIID_X]);
	    			a_y = ((double)mesg->acc[CWIID_Y] - wm_cal.zero[CWIID_Y]) / (wm_cal.one[CWIID_Y] - wm_cal.zero[CWIID_Y]);
		    		a_z = ((double)mesg->acc[CWIID_Z] - wm_cal.zero[CWIID_Z]) / (wm_cal.one[CWIID_Z] - wm_cal.zero[CWIID_Z]);
			    	a = sqrt(pow(a_x,2)+pow(a_y,2)+pow(a_z,2));
                    

    				roll = atan(a_x/a_z);
	    			if (a_z <= 0.0) {
		    			roll += PI * ((a_x > 0.0) ? 1 : -1);
			    	}
				    roll *= -1;

    				pitch = atan(a_y/a_z*cos(roll));

                    midivalue=(int)((((a>5.0)?5.0:(a<1.0)?0.0:a-1.0)/5.0)*(float)ppatch[patch_id].ev_CWIID_ACC[0].data.control.value);

                    if(accmidiold[0] != midivalue)
                    {
    	    			snd_seq_ev_clear(&ev);
	    	    		snd_seq_ev_set_controller(&ev, ppatch[patch_id].ev_CWIID_ACC[0].data.control.channel, ppatch[patch_id].ev_CWIID_ACC[0].data.control.param, midivalue);
		    	    	snd_seq_ev_set_source(&ev, port_out_id);
    		    		snd_seq_ev_set_subs(&ev);
	    		    	snd_seq_ev_set_direct(&ev);
		    		    snd_seq_event_output_direct(seq_handle, &ev);

                        accmidiold[0]=midivalue;
                    }

			    	//low pass filtering
    				if(roll>=-5 && roll<5)
	    			{
		    			pbRoll-=tabRoll[cptRoll];
			    		tabRoll[cptRoll]=roll;
					    pbRoll+=tabRoll[cptRoll];
				    	cptRoll=(cptRoll>=9)? 0 : cptRoll+1;
    					float tmpRoll = pbRoll/10.0;

                        midivalue=(int)((float)(tmpRoll+2.5)/5.0*(float)ppatch[patch_id].ev_CWIID_ACC[1].data.control.value);

                        if(accmidiold[1] != midivalue)
                        {
    	    				snd_seq_ev_clear(&ev);
	    	    			snd_seq_ev_set_controller(&ev, ppatch[patch_id].ev_CWIID_ACC[1].data.control.channel, ppatch[patch_id].ev_CWIID_ACC[1].data.control.param, midivalue);
		    	    		snd_seq_ev_set_source(&ev, port_out_id);
			    	    	snd_seq_ev_set_subs(&ev);
    			    		snd_seq_ev_set_direct(&ev);
	    			    	snd_seq_event_output_direct(seq_handle, &ev);

                            accmidiold[1]=midivalue;
                        }
		    		}

    				if(pitch>=-5 && pitch<5)
	    			{
		    			
				    	pbPitch-=tabPitch[cptPitch];
			    		tabPitch[cptPitch]=pitch;
    					pbPitch+=tabPitch[cptPitch];
	    				cptPitch=(cptPitch>=9)? 0 : cptPitch+1;
		    			float tmpPitch = pbPitch/10.0;

                        midivalue=(int)((float)(tmpPitch+1.5)/3.0*(float)ppatch[patch_id].ev_CWIID_ACC[1].data.control.value);
             
                        if(midivalue != accmidiold[2])
                        {
    			    		snd_seq_ev_clear(&ev);
	    			    	snd_seq_ev_set_controller(&ev, ppatch[patch_id].ev_CWIID_ACC[1].data.control.channel, ppatch[patch_id].ev_CWIID_ACC[1].data.control.param, midivalue);
		    			    snd_seq_ev_set_source(&ev, port_out_id);
    		    			snd_seq_ev_set_subs(&ev);
	    		    		snd_seq_ev_set_direct(&ev);
		    		    	snd_seq_event_output_direct(seq_handle, &ev);

                            accmidiold[2]=midivalue;
                        }
			    	}

                    snd_seq_ev_clear(&previous);
                    previous=ev;

                }
			}
                        break;
		case CWIID_MESG_NUNCHUK:
            if( ppatch[patch_id].nunchuk ) 
            {

				if(mesg_array[i].nunchuk_mesg.buttons & CWIID_NUNCHUK_BTN_Z && (previous.type != ppatch[patch_id].ev_CWIID_NUNCHUK_BTN_Z.type || previous.data.control.param != ppatch[patch_id].ev_CWIID_NUNCHUK_BTN_Z.data.control.param || previous.data.control.value != ppatch[patch_id].ev_CWIID_NUNCHUK_BTN_Z.data.control.value || previous.data.control.channel != ppatch[patch_id].ev_CWIID_NUNCHUK_BTN_Z.data.control.channel))
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_NUNCHUK_BTN_Z;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}

				if(mesg_array[i].nunchuk_mesg.buttons & CWIID_NUNCHUK_BTN_C && (previous.type != ppatch[patch_id].ev_CWIID_NUNCHUK_BTN_C.type || previous.data.control.param != ppatch[patch_id].ev_CWIID_NUNCHUK_BTN_C.data.control.param || previous.data.control.value != ppatch[patch_id].ev_CWIID_NUNCHUK_BTN_C.data.control.value || previous.data.control.channel != ppatch[patch_id].ev_CWIID_NUNCHUK_BTN_C.data.control.channel))
				{
					snd_seq_ev_clear(&ev);
                    ev=ppatch[patch_id].ev_CWIID_NUNCHUK_BTN_C;
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);
                    
                    snd_seq_ev_clear(&previous);
                    previous=ev;
				}
                else if((long int) mesg_array[i].nunchuk_mesg.stick)// && (previous.type != ppatch[patch_id].ev_CWIID_STICK.type || previous.data.control.param != ppatch[patch_id].ev_CWIID_STICK.data.control.param || previous.data.control.value != ppatch[patch_id].ev_CWIID_STICK.data.control.value || previous.data.control.channel != ppatch[patch_id].ev_CWIID_STICK.data.control.channel)) 
                {
	    			float s_x, s_y, s, max=sqrt(pow(210,2)+pow(210,2)), min=sqrt(pow(20,2)+pow(20,2));
                    int midivalue;

    				s_x = (mesg_array[i].nunchuk_mesg.stick[CWIID_X]-20);
	    			s_y = (mesg_array[i].nunchuk_mesg.stick[CWIID_Y]-20);
			    	s = sqrt(pow(s_x,2)+pow(s_y,2));
                    midivalue=(int)((s-min)*(float)ppatch[patch_id].ev_CWIID_STICK.data.control.value/max);

                    if(midiold != midivalue)
                    {
                        snd_seq_ev_clear(&ev);
	    	    		snd_seq_ev_set_controller(&ev, ppatch[patch_id].ev_CWIID_STICK.data.control.channel, ppatch[patch_id].ev_CWIID_STICK.data.control.param, midivalue);
		    	  		snd_seq_ev_set_source(&ev, port_out_id);
			        	snd_seq_ev_set_subs(&ev);
    			    	snd_seq_ev_set_direct(&ev);
	    			    snd_seq_event_output_direct(seq_handle, &ev);

                        snd_seq_ev_clear(&previous);
                        previous=ev;
                    }

                    midiold = midivalue;
                }
                
                else if((long int) mesg_array[i].nunchuk_mesg.acc && ppatch[patch_id].acc) {}



                if( previous.type==SND_SEQ_EVENT_NOTEON )
                {
                    //sends NOTE OFF if the previous event was a NOTE ON
					snd_seq_ev_clear(&ev);
					snd_seq_ev_set_noteoff(&ev, previous.data.note.channel, previous.data.note.note, 127);
					snd_seq_ev_set_source(&ev, port_out_id);
					snd_seq_ev_set_subs(&ev);
					snd_seq_ev_set_direct(&ev);
					snd_seq_event_output_direct(seq_handle, &ev);

                    snd_seq_ev_clear(&previous);
                    previous=ev;
                }
            }
			break;
		case CWIID_MESG_IR:
			break;
                default:
                        break;
                }

		  snd_seq_drain_output(seq_handle);
        }
    }
}


int main(int argc, char** argv)
{
	bdaddr = *BDADDR_ANY;


	if (snd_seq_open(&seq_handle, "default", SND_SEQ_OPEN_DUPLEX, 0) < 0)
	{
		cout<<"Error opening ALSA sequencer."<<endl;
		exit(1);
	}

	snd_seq_set_client_name(seq_handle, "Wiidi");
	if ((port_out_id = snd_seq_create_simple_port(seq_handle, "Wiidi",
		SND_SEQ_PORT_CAP_READ|SND_SEQ_PORT_CAP_SUBS_READ,
		SND_SEQ_PORT_TYPE_APPLICATION)) < 0 || (port_in_id = snd_seq_create_simple_port(seq_handle, "Wiidi",
        SND_SEQ_PORT_CAP_WRITE|SND_SEQ_PORT_CAP_SUBS_WRITE,
        SND_SEQ_PORT_TYPE_APPLICATION)) < 0) 
	{
		cout<<"Error creating sequencer port."<<endl;
	}

// Si fichier dans getopts
    string filename="test.txt";
    ifstream bindings(filename.c_str(),ios::in);
    if(bindings)
    {
        string bind;
        int pid;
        for( int i=0; i<16; i++ )
        {
            bindings.ignore(numeric_limits<int>::max(), '#');
            bindings >> bind >> pid;
            if( bind == "PatchNumber" && pid <16 )
            {

                cerr << "---------" << bind << " " << pid << "---------" << endl;

                //wiimote's acc, nunchuk and ir active or not?
                bindings >> bind >> ppatch[pid].acc;
                bindings >> bind >> ppatch[pid].nunchuk;
                bindings >> bind >> ppatch[pid].ir;
                cerr << ppatch[pid].acc << " Accéléromètre, " << ppatch[pid].nunchuk << " Nunchuk, " << ppatch[pid].ir << " Pointeur IR" << endl << endl;


                //MIDI conversion
                ppatch[pid].ev_CWIID_BTN_2=*assign_midi_event(bindings);
                ppatch[pid].ev_CWIID_BTN_1=*assign_midi_event(bindings);
                ppatch[pid].ev_CWIID_BTN_B=*assign_midi_event(bindings);
                ppatch[pid].ev_CWIID_BTN_A=*assign_midi_event(bindings);
                ppatch[pid].ev_CWIID_BTN_MINUS=*assign_midi_event(bindings);
                ppatch[pid].ev_CWIID_BTN_HOME=*assign_midi_event(bindings);
                ppatch[pid].ev_CWIID_BTN_LEFT=*assign_midi_event(bindings);
                ppatch[pid].ev_CWIID_BTN_RIGHT=*assign_midi_event(bindings);
                ppatch[pid].ev_CWIID_BTN_DOWN=*assign_midi_event(bindings);
                ppatch[pid].ev_CWIID_BTN_UP=*assign_midi_event(bindings);
                ppatch[pid].ev_CWIID_BTN_PLUS=*assign_midi_event(bindings);
     
                //if wiimote's nunchuk active in this patch
                if( ppatch[pid].nunchuk )
                {
                    ppatch[pid].ev_CWIID_NUNCHUK_BTN_Z=*assign_midi_event(bindings);
                    ppatch[pid].ev_CWIID_NUNCHUK_BTN_C=*assign_midi_event(bindings);
                    ppatch[pid].ev_CWIID_STICK=*assign_midi_event(bindings); // La valeur envoyée dans l'événement correspond au max
                }

                //if wiimote's acc active in this patch
                if( ppatch[pid].acc )
                {
                    ppatch[pid].ev_CWIID_ACC[0]=*assign_midi_event(bindings); // La valeur envoyée dans l'événement correspond au max
                    ppatch[pid].ev_CWIID_ACC[1]=*assign_midi_event(bindings); // La valeur envoyée dans l'événement correspond au max
                    ppatch[pid].ev_CWIID_ACC[2]=*assign_midi_event(bindings); // La valeur envoyée dans l'événement correspond au max
                }
                //if wiimote's ir active in this patch
                if( ppatch[pid].ir )
                    ppatch[pid].ev_CWIID_IR=*assign_midi_event(bindings); // La valeur envoyée dans l'événement correspond au max
            }
            else
                cerr << "!!!" << bind << ": " << pid << endl << endl;
        }
    }
    else
        cerr<<"Impossible d'ouvrir le fichier"<<endl;
    
// Sinon


	for(int i=0;i<10;++i)
	{
		tabRoll[i]=0;
		tabPitch[i]=0;
	}

	cout<<"Press 1+2" <<endl;

        if ((wiimote = cwiid_open(&bdaddr, CWIID_FLAG_MESG_IFC)) == NULL)
                cout<<"Unable to connect"<<endl;
	else if(cwiid_set_mesg_callback(wiimote,&cwiid_callback))
	{
		cout<<"Error setting callback"<<endl;
		if(cwiid_close(wiimote))
			cout<<"Error on disconnect"<<endl;
	}
        else {
                if (cwiid_get_acc_cal(wiimote, CWIID_EXT_NONE, &wm_cal))
                        cout<<"Unable to retrieve accelerometer calibration"<<endl;
                if (cwiid_get_acc_cal(wiimote, CWIID_EXT_NUNCHUK, &nc_cal))
                        cout<<"Unable to retrieve nunchuck calibration"<<endl;
                cwiid_set_rpt_mode(wiimote,  CWIID_RPT_BTN | CWIID_RPT_ACC | CWIID_RPT_EXT|CWIID_RPT_IR);
        }

    npfd = snd_seq_poll_descriptors_count(seq_handle, POLLIN);
    pfd = (struct pollfd *)alloca(npfd * sizeof(struct pollfd));
    snd_seq_poll_descriptors(seq_handle, pfd, npfd, POLLIN);

	while(1){
        if(poll(pfd, npfd, 100000) > 0) {
            midi_action(seq_handle);
        }
    }
	return 0;
}


